package MyTest::DBIC::Complex::Code;

use strict;

use base 'MyTest::DBIC::Base';

__PACKAGE__->table('rose_db_object_test_codes');
__PACKAGE__->add_columns(qw(code k1 k2 k3));
__PACKAGE__->set_primary_key('k1', 'k2', 'k3');

1;